from typing import Any, Callable, List, Union


class Node:
    def __init__(
        self,
        default_output: Any = None,
        default_state: bool = True,
        name: str = None,
        ravel: bool = False,
        *args: Any,
        **kwargs: Any,
    ):
        self._output = default_output
        self._state = default_state
        self.name = name
        self.ravel = ravel
        self._memory = kwargs

    @property
    def name(self) -> str:
        return self._name

    @name.setter
    def name(self, name: str) -> None:
        self._name = self.__class__.__name__ if name is None else name

    @property
    def ravel(self) -> str:
        return self._ravel

    @ravel.setter
    def ravel(self, ravel: bool) -> None:
        self._ravel = ravel

    @property
    def state(self) -> bool:
        return self._state

    @property
    def output(self) -> Any:
        return self._output

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        raise NotImplementedError


class NullNode(Node):
    def __call__(self, *args: Any, **kwargs: Any) -> None:
        return None


class ConstantNode(Node):
    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        return self._output


class WrapperNode(Node):
    def __init__(self, func: Callable, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        self._func = func
        self.__call__.__func__.__doc__ = func.__doc__

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        try:
            self._output = self._func(*args, **kwargs, **self._memory)
            self._state = True
        except:
            self._output = None
            self._state = False
        return self._output


class NodeSet(Node):
    def __init__(
        self,
        nodes: Union[Node, List[Node]] = [],
        *args: Any,
        **kwargs: Any,
    ) -> None:
        super().__init__(*args, **kwargs)
        self.nodes = nodes

    @property
    def nodes(self) -> List[Node]:
        return self._nodes

    @nodes.setter
    def nodes(self, nodes: Union[Node, List[Node]]) -> None:
        if isinstance(nodes, Node):
            nodes = [nodes]

        self._nodes = []
        for node in nodes:
            self.add_node(node)

    def add_node(self, node: Node, name: str = None) -> "NodeSet":
        if not isinstance(node, Node):
            node = WrapperNode(node)

        if name is not None:
            node.name = name

        self._nodes.append(node)
        return self

    def __call__(self, *args: Any, **kwargs: Any) -> None:
        for node in self.nodes:
            print(f"Running: {node.name}")
            node()


class NodeSequence(NodeSet):
    def __init__(
        self,
        nodes: Union[Node, List[Node]] = [],
        ignore_none_output: bool = True,
        *args: Any,
        **kwargs: Any,
    ):
        super().__init__(nodes, *args, **kwargs)
        self._ignore_none_output = ignore_none_output

    @property
    def ignore_none_output(self) -> bool:
        return self._ignore_none_output

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        result = None
        first = True
        ravel = False
        for node in self.nodes:
            print(f"Running: {node.name}")
            if first:
                result = node(*args, **kwargs, **self._memory)
                first = False
            elif result is None and self.ignore_none_output:
                result = node(**self._memory)
            elif ravel:
                result = node(**result, **self._memory)
            else:
                result = node(result, **self._memory)

            self._state = node.state
            self._output = node.output
            ravel = node.ravel

            if not node.state:
                return self.output

        return self.output


class UnionNode(NodeSequence):
    def __init__(
        self,
        nodes: Union[Node, List[Node]] = [],
        return_node_names: Union[str, List[str]] = None,
        *args: Any,
        **kwargs: Any,
    ):
        super().__init__(nodes=nodes, *args, **kwargs)
        self.return_node_names = return_node_names

    @property
    def return_node_names(self) -> List[Node]:
        return self._return_node_names

    @return_node_names.setter
    def return_node_names(self, return_node_names: Union[str, List[str]]) -> None:
        if isinstance(return_node_names, str):
            return_node_names = [return_node_names]

        self._return_node_names = return_node_names

    @property
    def output(self) -> Any:
        if self.return_node_names is None:
            return self._output
        elif len(self.return_node_names) == 1:
            if self.return_node_names[0] not in self._output:
                return None
            return self._output[self.return_node_names[0]]
        return {index: self._output[index] for index in self.return_node_names}

    def __call__(
        self,
        *args: Any,
        **kwargs: Any,
    ) -> Any:
        self._output = {}
        self._state = True
        for node in self.nodes:
            print(f"Running: {node.name}")
            result = node(*args, **kwargs, **self._memory)

            if not (result is None and self.ignore_none_output):
                self._output[node.name] = result

            if not node.state:
                self._output = None
                self._state = False
                return None

        return self.output


class BranchNode(Node):
    def __init__(
        self,
        trigger: Node,
        positive: Node = NullNode(),
        negative: Node = NullNode(),
        ignore_none_output: bool = True,
        *args: Any,
        **kwargs: Any,
    ):
        super().__init__(*args, **kwargs)
        self.trigger = trigger
        self.positive = positive
        self.negative = negative
        self._ignore_none_output = ignore_none_output

    @property
    def trigger(self) -> Node:
        return self._trigger

    @property
    def positive(self) -> Node:
        return self._positive

    @property
    def negative(self) -> Node:
        return self._negative

    @property
    def ignore_none_output(self) -> bool:
        return self._ignore_none_output

    @trigger.setter
    def trigger(self, trigger: Node) -> None:
        if not isinstance(trigger, Node):
            trigger = WrapperNode(trigger)

        self._trigger = trigger

    @positive.setter
    def positive(self, positive: Node) -> None:
        if not isinstance(positive, Node):
            positive = WrapperNode(positive)

        self._positive = positive

    @negative.setter
    def negative(self, negative: Node) -> None:
        if not isinstance(negative, Node):
            negative = WrapperNode(negative)

        self._negative = negative

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        trigger_result = self.trigger(*args, **kwargs, **self._memory)

        if self.trigger.state:
            if trigger_result is None and self.ignore_none_output:
                self._output = self.positive(**self._memory)
            elif self.trigger.ravel:
                self._output = self.positive(**trigger_result, **self._memory)
            else:
                self._output = self.positive(trigger_result, **self._memory)
            self._state = self.positive.state
        else:
            if trigger_result is None and self.ignore_none_output:
                self._output = self.negative(**self._memory)
            elif self.trigger.ravel:
                self._output = self.negative(**trigger_result, **self._memory)
            else:
                self._output = self.negative(trigger_result, **self._memory)
            self._state = self.negative.state

        return self.output



from typing import Any

try:
    import cv2
    import numpy as np
except ImportError:
    raise ImportError(
        "cv2 is not installed. Please install it with `pip install opencv-python`."
    )

try:
    import mss
except ImportError:
    raise ImportError("mss is not installed. Please install it with `pip install mss`.")

try:
    import pyautogui
except ImportError:
    raise ImportError(
        "pyautogui is not installed. Please install it with `pip install pyautogui`."
    )

from gurun.node import Node


class ScreenshotPAG(Node):
    def __call__(self, filename: str = None, *args: Any, **kwargs: Any) -> np.ndarray:
        image = pyautogui.screenshot()

        if filename is not None:
            image.save(filename)

        self._output = cv2.cvtColor(np.array(image), cv2.COLOR_RGB2BGR)
        return self.output


class ScreenshotMMS(Node):
    def __init__(self, monitor: int = 0, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self._monitor = monitor

    def __call__(self, filename: str = None, *args: Any, **kwargs: Any) -> np.ndarray:

        with mss.mss() as sct:
            self._output = np.array(sct.grab(sct.monitors[self._monitor]))[:, :, :3]

        if filename is not None:
            cv2.imwrite(filename, self.output)

        return self.output



from typing import Any, List, Union

try:
    import cv2
    import numpy as np
except ImportError:
    raise ImportError(
        "cv2 is not installed. Please install it with `pip install opencv-python`."
    )

from gurun.node import Node


class TemplateDetection(Node):
    def __init__(
        self,
        target: Union[np.ndarray, str],
        threshold: float = 0.7,
        single_match: bool = False,
        method: int = cv2.TM_CCOEFF_NORMED,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        super().__init__(*args, **kwargs)
        if isinstance(target, str):
            target = cv2.imread(target)
            if target is None:
                raise ValueError(
                    f"Template Detection target file {target} does not exist"
                )

        self._target = target
        self._target_height = int(target.shape[0])
        self._target_width = int(target.shape[1])
        self._threshold = threshold
        self._single_match = single_match
        self._method = method

    def __call__(
        self, image: Union[np.ndarray, str], *args: Any, **kwargs: Any
    ) -> List[List[int]]:
        if isinstance(image, str):
            image = cv2.imread(image)
            if image is None:
                raise ValueError("Template Detection image file does not exist")

        result = cv2.matchTemplate(image, self._target, self._method)

        yloc, xloc = np.where(result >= self._threshold)

        rectangles = []
        for (x, y) in zip(xloc, yloc):
            rectangles.append([int(x), int(y), self._target_width, self._target_height])
            rectangles.append([int(x), int(y), self._target_width, self._target_height])

        rectangles, _ = cv2.groupRectangles(rectangles, 1, 0.2)

        if len(rectangles) == 0:
            self._output = None
            self._state = False
        else:
            self._output = rectangles[0] if self._single_match else rectangles
            self._state = True

        return self._output


class TemplateDetectionFrom(TemplateDetection):
    def __init__(
        self,
        source_node: Node,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        super().__init__(*args, **kwargs)
        self._source_node = source_node

    def __call__(self, *args: Any, **kwargs: Any) -> List[List[int]]:
        image = self._source_node(*args, **kwargs)

        return super().__call__(image, *args, **kwargs)



from typing import Any

import subprocess

from gurun import Node


class Subprocess(Node):
    def __init__(self, *args: Any, **kwargs: Any):
        super().__init__(**kwargs)
        self._commands = args

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        self._state = True
        self._output = subprocess.run(self._commands, *args, **kwargs, **self._memory)
        return self._output


class Workspace(Subprocess):
    def __init__(self, workspace: str, os: str, *args: Any, **kwargs: Any):
        if os.lower() == "linux":
            super().__init__("wmctrl", "-s", workspace, *args, **kwargs)
        else:
            raise ValueError("Workspace is only available on Linux")



from typing import List

from gurun.exceptions import RunnerException
from gurun.node import BranchNode, Node, NodeSet, NullNode
from gurun.utils import RaiseException, Sleep


class Runner(NodeSet):
    def __init__(
        self,
        nodes: List[Node],
        start_node: Node = NullNode(),
        end_node: Node = NullNode(),
        interval: int = 5,
        *args,
        **kwargs,
    ) -> None:
        super().__init__(nodes, *args, **kwargs)

        self._interval_node = Sleep(interval)
        self._start_node = BranchNode(
            start_node,
            negative=RaiseException(
                RunnerException("Could not successfully run start node")
            ),
        )
        self._end_node = BranchNode(
            end_node,
            negative=RaiseException(
                RunnerException("Could not successfully run end node")
            ),
        )

    def __call__(self):
        self._start_node()

        try:
            while True:
                for node in self.nodes:
                    node()

                    self._interval_node()

        except KeyboardInterrupt:
            print("Interrupted!")

        self._end_node()



from typing import Any, Dict, List

import math
import random

from gurun.node import Node

try:
    import cv2
    import numpy as np
except ImportError:
    raise ImportError(
        "cv2 is not installed. Please install it with `pip install opencv-python`."
    )


class Transformation(Node):
    def __call__(self, detections: np.ndarray, *args: Any, **kwargs: Any) -> Any:
        if detections is None:
            self._output = None
            self._state = False
            return None

        self._output = self._transform(detections, *args, **kwargs)
        self._state = True

        return self.output

    def _transform(self, detections: np.ndarray, *args: Any, **kwargs: Any) -> Any:
        raise NotImplementedError()


class RectToPoint(Transformation):
    def _transform(self, detections: np.ndarray, *args: Any, **kwargs: Any) -> Any:
        if len(np.shape(detections)) == 1:
            return {
                "x": detections[0] + (detections[2] / 2),
                "y": detections[1] + (detections[3] / 2),
            }
        elif len(np.shape(detections)) == 2:
            return [[r[0] + (r[2] / 2), r[1] + (r[3] / 2)] for r in detections]
        else:
            raise ValueError(
                "RectToPoint: Input must be a 1D or 2D array. Input shape: {}".format(
                    np.shape(detections)
                )
            )


class NaturalRectToPoint(Transformation):
    def __init__(self, border_proportion: float = 0.25, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        self._border_proportion = border_proportion

    def __natural_range(self, value: int, limit: int) -> int:
        value_start = value + (limit * self._border_proportion)
        value_end = (value + limit) - (limit * self._border_proportion)
        return math.floor(random.uniform(value_start, value_end))

    def _transform(self, detections: np.ndarray, *args: Any, **kwargs: Any) -> Any:
        if len(np.shape(detections)) == 1:
            return {
                "x": self.__natural_range(detections[0], detections[2]),
                "y": self.__natural_range(detections[1], detections[3]),
            }
        elif len(np.shape(detections)) == 2:
            return [
                [self.__natural_range(r[0], r[2]), self.__natural_range(r[1], r[3])]
                for r in detections
            ]
        else:
            raise ValueError(
                "NaturalRectToPoint: Input must be a 1D or 2D array. Input shape: {}".format(
                    np.shape(detections)
                )
            )


class Offset(Transformation):
    def __init__(self, xOffset: int = 0, yOffset: int = 0, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._xOffset = xOffset
        self._yOffset = yOffset

    def _transform(self, detections: np.ndarray, *args: Any, **kwds: Any) -> Any:
        if isinstance(detections, dict):
            return {
                "x": detections["x"] + self._xOffset,
                "y": detections["y"] + self._yOffset,
            }
        elif len(np.shape(detections)) == 1:
            detections[0] += self._xOffset
            detections[1] += self._yOffset
            return detections
        elif len(np.shape(detections)) == 2:
            for detection in detections:
                detection[0] += self._xOffset
                detection[1] += self._yOffset

            return detections
        else:
            raise ValueError(
                "Offset: Input must be a 1D or 2D array. Input shape: {}".format(
                    np.shape(detections)
                )
            )



from typing import Any, Callable, List

import random

from gurun.node import Node, WrapperNode

try:
    import pyautogui
except ImportError:
    raise ImportError(
        "pyautogui is not installed. Please install it with `pip install pyautogui`."
    )


class Typewrite(WrapperNode):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(pyautogui.typewrite, *args, **kwargs)


class Scroll(Node):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(pyautogui.scroll, *args, **kwargs)


class Click(WrapperNode):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(pyautogui.click, *args, **kwargs)


class HotKey(WrapperNode):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(pyautogui.hotkey, *args, **kwargs)


class MoveRel(Node):
    def __init__(
        self,
        x: int = 0,
        y: int = 0,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        super().__init__(*args, **kwargs)
        self._x = x
        self._y = y

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        pyautogui.moveRel(self._x, self._y, **self._memory)


class MoveTo(WrapperNode):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(pyautogui.moveTo, *args, **kwargs)


class DragRel(WrapperNode):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(pyautogui.dragRel, *args, **kwargs)


class MultipleClicks(Click):
    def __call__(self, positions: List[List[int]], *args: Any, **kwargs: Any):
        for x, y in positions:
            super().__call__(*args, x=x, y=y, **kwargs)


class NaturalClick(Click):
    def __init__(
        self,
        easing_functions: List[Callable] = [
            pyautogui.easeInQuad,
            pyautogui.easeOutQuad,
            pyautogui.easeInOutQuad,
        ],
        minimum_duration: int = 1,
        maximum_duration: int = 1.5,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        super().__init__(*args, **kwargs)
        self._easing_functions = easing_functions
        self._minimum_duration = minimum_duration
        self._maximum_duration = maximum_duration

    def __call__(self, *args: Any, **kwargs: Any):
        return super().__call__(
            *args,
            tween=random.choice(self._easing_functions),
            duration=random.uniform(self._minimum_duration, self._maximum_duration),
            **kwargs,
        )


class MultipleNaturalClicks(NaturalClick):
    def __call__(self, positions: List[List[int]], *args: Any, **kwargs: Any):
        for x, y in positions:
            super().__call__(*args, x=x, y=y, **kwargs)



"""Framework de autotomação de tarefas"""

import sys

if sys.version_info >= (3, 8):
    from importlib import metadata as importlib_metadata
else:
    import importlib_metadata  # pragma: no cover


def get_version() -> str:
    try:
        return importlib_metadata.version(__name__)
    except importlib_metadata.PackageNotFoundError:  # pragma: no cover
        return "unknown"


version: str = get_version()

from gurun import exceptions, runner, utils
from gurun.node import (
    BranchNode,
    ConstantNode,
    Node,
    NodeSequence,
    NodeSet,
    NullNode,
    UnionNode,
    WrapperNode,
)

__all__ = [
    "version",
    "Node",
    "NullNode",
    "ConstantNode",
    "WrapperNode",
    "NodeSet",
    "NodeSequence",
    "UnionNode",
    "BranchNode",
    "exceptions",
    "runner",
    "utils",
]



from typing import Any

import random
import time

from gurun.node import Node, WrapperNode


class Sleep(Node):
    def __init__(self, interval: int = 5, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        self._interval = interval

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        time.sleep(self._interval)


class RaiseException(Node):
    def __init__(self, exception: Exception, *args: Any, **kwargs: Any):
        super().__init__(default_state=False, *args, **kwargs)
        self._exception = exception

    def __call__(self, *args: Any, **kwargs: Any) -> None:
        raise self._exception


class Periodic(Node):
    def __init__(self, node: Node, interval: int, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        self._node = node
        self._interval = interval
        self._last_run = 0

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        if time.time() - self._last_run < self._interval:
            self._state = False
            return None

        self._output = self._node(*args, **kwargs, **self._memory)
        self._state = self._node.state
        if self.state:
            self._last_run = time.time()

        return self._output


class RandomPeriodic(Periodic):
    def __init__(
        self,
        node: Node,
        min_interval: int,
        max_interval: int,
        *args: Any,
        **kwargs: Any,
    ):
        super().__init__(node, 0, *args, **kwargs)
        self._min_interval = min_interval
        self._max_interval = max_interval

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        self._interval = random.randint(self._min_interval, self._max_interval)
        return super().__call__(*args, **kwargs)


class Wait(Node):
    def __init__(self, node: Node, timeout: int, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        self._node = node
        self._timeout = timeout

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        start = time.time()
        while time.time() - start < self._timeout:
            self._node(*args, **kwargs, **self._memory)
            if self._node.state:
                self._state = True
                self._output = self._node.output
                return self._node.output

        self._state = False
        return None


class NotNode(Node):
    def __init__(self, node: Node, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        self._node = node

    def __call__(self, *args: Any, **kwargs: Any) -> bool:
        self._output = self._node(*args, **kwargs, **self._memory)
        self._state = not self._node.state
        return self.output


class While(Node):
    def __init__(
        self, trigger: Node, action: Node, timeout: int, *args: Any, **kwargs: Any
    ):
        super().__init__(*args, **kwargs)
        self._trigger = trigger
        self._action = action
        self._timeout = timeout

    def __call__(self, *args: Any, **kwargs: Any) -> None:
        start = time.time()
        self._state = False
        while time.time() - start < self._timeout:
            self._trigger(*args, **kwargs, **self._memory)
            if self._trigger.state:
                self._action(*args, **kwargs, **self._memory)
            else:
                self._state = True
                break


class Print(WrapperNode):
    def __init__(self, *args: Any, **kwargs: Any):
        super().__init__(print, *args, **kwargs)
        self._args = args

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        return super().__call__(*self._args, *args, **kwargs)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gurun', 'gurun.cv', 'gurun.gui']

package_data = \
{'': ['*']}

extras_require = \
{'cv': ['opencv-python>=4.5.4,<5.0.0'],
 'full': ['opencv-python>=4.5.4,<5.0.0',
          'PyAutoGUI>=0.9.41,<0.10.0',
          'mss>=6.1.0,<7.0.0'],
 'gui': ['opencv-python>=4.5.4,<5.0.0',
         'PyAutoGUI>=0.9.41,<0.10.0',
         'mss>=6.1.0,<7.0.0']}

setup_kwargs = {
    'name': 'gurun',
    'version': '0.5.0',
    'description': 'Task automation framework',
    'long_description': '# gurun\n\n<div align="center">\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/gabrielguarisa/gurun/blob/master/.pre-commit-config.yaml)\n[![Semantic Versions](https://img.shields.io/badge/%20%20%F0%9F%93%A6%F0%9F%9A%80-semantic--versions-e10079.svg)](https://github.com/gabrielguarisa/gurun/releases)\n[![License](https://img.shields.io/github/license/gabrielguarisa/gurun)](https://github.com/gabrielguarisa/gurun/blob/master/LICENSE)\n\nTask automation framework\n\n</div>\n\n## Installation\n\n```bash\npip install gurun\n```\n\n## Development\n### Setting up a development environment\n\nIf you don\'t have a local development environment, you can follow these steps to set one up.\n\nFirst, if you have not already, install [poetry](https://python-poetry.org/):\n\n```bash\npip install poetry\n```\n\nNow, initialize poetry and [pre-commit](https://pre-commit.com/) hooks:\n\n```bash\nmake install && make install-pre-commit\n```\n\n### Running tests\n\nYou can run the tests with:\n\n```bash\nmake tests\n```\n\nThis will run the tests with [pytest](https://docs.pytest.org/en/latest/) and show information about the coverage.\n\n### Formatting the code\n\nTo format the code, you can use the command:\n\n```bash\nmake formatting\n```\n\nThis will run the [black](https://github.com/psf/black), [isort](https://github.com/PyCQA/isort) and )[pyupgrade](https://github.com/asottile/pyupgrade) commands.\n\nIf you want to just check the formatting, use the command:\n\n```bash\nmake check-formatting\n```\n\n### Releasing a new version\n\nTo release a new version, you need to follow these steps:\n\n1. Update the version with `poetry version <version>` and commit the changes. This project follows [Semantic Versioning](http://semver.org/), so the version number should follow the format `<major>.<minor>.<patch>`. Alternatively, you can also use the version as `major` or `minor` or `patch`, and the version number will be automatically incremented.\n\n2. Create a Github release with the new version number.\n\n3. (Optional) Publish the new version to PyPI with `poetry publish --build`.\n',
    'author': 'Gabriel Guarisa',
    'author_email': 'gabrielguarisa@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/gabrielguarisa/gurun',
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'python_requires': '>=3.7.12,<3.11',
}


setup(**setup_kwargs)
